@@ -23,6 +23,12 @@
 
 #ifdef OTFMERGE_MPI
 #   include "mpi.h"
+#   if defined(INSIDE_OPENMPI) || (defined(HAVE_MPI_GET_ADDRESS) && HAVE_MPI_GET_ADDRESS)
+#       define MPI_Address MPI_Get_address
+#   endif /* HAVE_MPI_GET_ADDRESS */
+#   if defined(INSIDE_OPENMPI) || defined(HAVE_MPI_TYPE_CREATE_STRUCT) && HAVE_MPI_TYPE_CREATE_STRUCT
+#       define MPI_Type_struct MPI_Type_create_struct
+#   endif /* HAVE_MPI_TYPE_CREATE_STRUCT */
 #endif /* OTFMERGE_MPI */
 
 #include "otf.h"
@@ -130,6 +136,8 @@ int handleDefSclFile( void *userData, uint32_t stream, uint32_t sourceFile,
 int handleDefCreator( void *userData, uint32_t stream, const char *creator,
         OTF_KeyValueList *list );
 
+int handleDefUniqueId( void *userData, uint32_t stream, uint64_t uid );
+
 int handleDefVersion( void *userData, uint32_t stream, uint8_t major,
         uint8_t minor, uint8_t sub, const char *string );
 
@@ -154,9 +162,18 @@ int handleDefProcessSubstitutes( void* userData, uint32_t stream,
         uint32_t representative, uint32_t numberOfProcs, const uint32_t* procs,
         OTF_KeyValueList *list );
 
+int handleDefAuxSamplePoint( void *fcbx,
+                             uint32_t               streamid,
+                             uint64_t               time,
+                             OTF_AuxSamplePointType type,
+                             OTF_KeyValueList*      list );
+
 int handleDefMarker( void *userData, uint32_t stream, uint32_t token,
         const char *name, uint32_t type, OTF_KeyValueList *list );
 
+int handleMarker( void *userData, uint64_t time, uint32_t process,
+        uint32_t token, const char* text, OTF_KeyValueList* list );
+
 int handleUnknownRecord( void *userData, uint64_t time, uint32_t process,
         const char *record );
 
