@@ -1,5 +1,5 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2011.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2012.
  Authors: Andreas Knuepfer, Robert Dietrich, Matthias Jurenz
 */
 
@@ -34,21 +34,30 @@ static char* pack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
 
         sizes[1]= alldata.functionMapGlobal.size(); /* map< uint64_t, FunctionData > functionMapGlobal; */
         num_fences++;
-        sizes[2]= alldata.counterMapGlobal.size(); /* map< Pair, CounterData, ltPair > counterMapGlobal; */
+        sizes[2]= alldata.functionDurationSectionMapGlobal.size(); /* map< Pair, FunctionData > functionDurationSectionMapGlobal; */
         num_fences++;
-        sizes[3]= alldata.messageMapPerGroupPair.size(); /* map< Pair, MessageData, ltPair > messageMapPerGroupPair; */
+        sizes[3]= alldata.counterMapGlobal.size(); /* map< Pair, CounterData, ltPair > counterMapGlobal; */
         num_fences++;
-        sizes[4]= alldata.messageMapPerGroup.size(); /* map< uint64_t, MessageData > messageMapPerGroup; */
+        sizes[4]= alldata.messageMapPerGroupPair.size(); /* map< Pair, MessageData, ltPair > messageMapPerGroupPair; */
         num_fences++;
-        sizes[5]= alldata.messageSpeedMapPerLength.size(); /* map< Pair, MessageSpeedData, ltPair > messageSpeedMapPerLength; */
+        sizes[5]= alldata.messageMapPerGroup.size(); /* map< uint64_t, MessageData > messageMapPerGroup; */
         num_fences++;
-        sizes[6]= alldata.collectiveMapPerGroup.size(); /* map< Pair, CollectiveData, ltPair > collectiveMapPerGroup; */
+        sizes[6]= alldata.messageSpeedMapPerLength.size(); /* map< Pair, MessageSpeedData, ltPair > messageSpeedMapPerLength; */
         num_fences++;
+        sizes[7]= alldata.collectiveMapPerGroup.size(); /* map< Pair, CollectiveData, ltPair > collectiveMapPerGroup; */
+        num_fences++;
+
     }
 
     if ( alldata.params.clustering.enabled ) {
 
-        sizes[7]= alldata.functionMapPerRank.size(); /* map< Pair, FunctionData, ltPair > */
+        sizes[8]= alldata.functionMapPerRank.size(); /* map< Pair, FunctionData, ltPair > */
+        num_fences++;
+    }
+    
+    if ( alldata.params.create_marker ) {
+        
+        sizes[9]= alldata.functionMinMaxLocationMap.size(); /* map< uint64_t, FunctionMinMaxLocactionData > functionMinMaxLocationMap; */
         num_fences++;
     }
 
@@ -67,26 +76,34 @@ static char* pack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
     MPI_Pack_size( sizes[2] * 8, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
     MPI_Pack_size( sizes[2] * 6, MPI_DOUBLE, MPI_COMM_WORLD, &s2 );
     bytesize += s1 + s2;
-
-    MPI_Pack_size( sizes[3] * 20, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
+    
+    MPI_Pack_size( sizes[3] * 8, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
     MPI_Pack_size( sizes[3] * 6, MPI_DOUBLE, MPI_COMM_WORLD, &s2 );
     bytesize += s1 + s2;
 
-    MPI_Pack_size( sizes[4] * 19, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
+    MPI_Pack_size( sizes[4] * 20, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
     MPI_Pack_size( sizes[4] * 6, MPI_DOUBLE, MPI_COMM_WORLD, &s2 );
     bytesize += s1 + s2;
 
-    MPI_Pack_size( sizes[5] * 6, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
-    bytesize += s1;
-
-    MPI_Pack_size( sizes[6] * 20, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
-    MPI_Pack_size( sizes[6] * 6, MPI_DOUBLE, MPI_COMM_WORLD, &s2 );
+    MPI_Pack_size( sizes[5] * 19, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
+    MPI_Pack_size( sizes[5] * 6, MPI_DOUBLE, MPI_COMM_WORLD, &s2 );
     bytesize += s1 + s2;
 
-    MPI_Pack_size( sizes[7] * 8, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
+    MPI_Pack_size( sizes[6] * 6, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
+    bytesize += s1;
+
+    MPI_Pack_size( sizes[7] * 20, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
     MPI_Pack_size( sizes[7] * 6, MPI_DOUBLE, MPI_COMM_WORLD, &s2 );
     bytesize += s1 + s2;
 
+    MPI_Pack_size( sizes[8] * 8, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
+    MPI_Pack_size( sizes[8] * 6, MPI_DOUBLE, MPI_COMM_WORLD, &s2 );
+    bytesize += s1 + s2;
+
+    MPI_Pack_size( sizes[9] * 7, MPI_LONG_LONG_INT, MPI_COMM_WORLD, &s1 );
+    MPI_Pack_size( sizes[9] * 0, MPI_DOUBLE, MPI_COMM_WORLD, &s2 );
+    bytesize += s1 + s2;
+    
     /* get the buffer */
     sizes[0]= bytesize;
     char* buffer= alldata.guaranteePackBuffer( bytesize );
@@ -105,7 +122,7 @@ static char* pack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
             map< uint64_t, FunctionData >::const_iterator itend= alldata.functionMapGlobal.end();
             for ( ; it != itend; ++it ) {
 
-                MPI_Pack( (void*) &it->first,            1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->first,                1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
 
                 MPI_Pack( (void*) &it->second.count.min,     1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
                 MPI_Pack( (void*) &it->second.count.max,     1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
@@ -128,6 +145,37 @@ static char* pack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
         /* extra check that doesn't cost too much */
         MPI_Pack( (void*) &fence, 1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
 
+        /* pack functionDurationSectionMapGlobal */
+        {
+            map< Pair, FunctionData, ltPair >::const_iterator it=    alldata.functionDurationSectionMapGlobal.begin();
+            map< Pair, FunctionData, ltPair >::const_iterator itend= alldata.functionDurationSectionMapGlobal.end();
+            for ( ; it != itend; ++it ) {
+                
+                MPI_Pack( (void*) &it->first.a,              1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->first.b,              1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+                
+                MPI_Pack( (void*) &it->second.count.min,     1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->second.count.max,     1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->second.count.sum,     1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->second.count.cnt,     1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+                
+                MPI_Pack( (void*) &it->second.excl_time.min, 1, MPI_DOUBLE,        buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->second.excl_time.max, 1, MPI_DOUBLE,        buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->second.excl_time.sum, 1, MPI_DOUBLE,        buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->second.excl_time.cnt, 1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+                
+                MPI_Pack( (void*) &it->second.incl_time.min, 1, MPI_DOUBLE,        buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->second.incl_time.max, 1, MPI_DOUBLE,        buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->second.incl_time.sum, 1, MPI_DOUBLE,        buffer, bytesize, &position, MPI_COMM_WORLD );
+                MPI_Pack( (void*) &it->second.incl_time.cnt, 1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+            }
+            alldata.functionDurationSectionMapGlobal.clear();           
+            
+        }
+
+        /* extra check that doesn't cost too much */
+        MPI_Pack( (void*) &fence, 1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+        
         /* pack counterMapGlobal */
         {
             map< Pair, CounterData, ltPair >::const_iterator it=    alldata.counterMapGlobal.begin();
@@ -350,6 +398,28 @@ static char* pack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
         /* extra check that doesn't cost too much */
         MPI_Pack( (void*) &fence, 1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
     }
+    
+    if ( alldata.params.create_marker ) {
+        
+        /* pack functionMinMaxLocationMap */
+        
+        map<uint64_t, FunctionMinMaxLocationData>::const_iterator it= alldata.functionMinMaxLocationMap.begin();
+        map<uint64_t, FunctionMinMaxLocationData>::const_iterator itend= alldata.functionMinMaxLocationMap.end();
+        
+        for ( ; it != itend ; ++it ) {
+            
+            MPI_Pack( (void*) &it->first,                    1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+            MPI_Pack( (void*) &it->second.location.min,      1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+            MPI_Pack( (void*) &it->second.location.max,      1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+            MPI_Pack( (void*) &it->second.location.loc_min,  1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+            MPI_Pack( (void*) &it->second.location.loc_max,  1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+            MPI_Pack( (void*) &it->second.location.time_max, 1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+            MPI_Pack( (void*) &it->second.location.time_max, 1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+        }
+        
+        /* extra check that doesn't cost too much */
+        MPI_Pack( (void*) &fence, 1, MPI_LONG_LONG_INT, buffer, bytesize, &position, MPI_COMM_WORLD );
+    }
 
     return buffer;
 }
@@ -410,8 +480,41 @@ static void unpack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
         MPI_Unpack( buffer, sizes[0], &position, &fence, 1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
         assert( FENCE == fence );
 
-        /* unpack counterMapGlobal */
+        /* unpack functionDurationSectionMapGlobal */
         for ( uint32_t i= 0; i < sizes[2]; i++ ) {
+            
+            uint64_t func;
+            uint64_t bin;
+            FunctionData tmp;
+            
+            MPI_Unpack( buffer, sizes[0], &position, &func,              1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &bin,               1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.count.min,     1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.count.max,     1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.count.sum,     1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.count.cnt,     1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.excl_time.min, 1, MPI_DOUBLE,        MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.excl_time.max, 1, MPI_DOUBLE,        MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.excl_time.sum, 1, MPI_DOUBLE,        MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.excl_time.cnt, 1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.incl_time.min, 1, MPI_DOUBLE,        MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.incl_time.max, 1, MPI_DOUBLE,        MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.incl_time.sum, 1, MPI_DOUBLE,        MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, &tmp.incl_time.cnt, 1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            
+            alldata.functionDurationSectionMapGlobal[ Pair( func, bin ) ].add( tmp );
+        }
+        
+        /* extra check that doesn't cost too much */
+        fence= 0;
+        MPI_Unpack( buffer, sizes[0], &position, &fence, 1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+        assert( FENCE == fence );
+        
+        /* unpack counterMapGlobal */
+        for ( uint32_t i= 0; i < sizes[3]; i++ ) {
 
             uint64_t a;
             uint64_t b;
@@ -444,7 +547,7 @@ static void unpack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
         assert( FENCE == fence );
 
         /* unpack messageMapPerGroupPair */
-        for ( uint32_t i= 0; i < sizes[3]; i++ ) {
+        for ( uint32_t i= 0; i < sizes[4]; i++ ) {
 
             uint64_t a;
             uint64_t b;
@@ -492,7 +595,7 @@ static void unpack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
         assert( FENCE == fence );
 
         /* unpack messageMapPerGroup */
-        for ( uint32_t i= 0; i < sizes[4]; i++ ) {
+        for ( uint32_t i= 0; i < sizes[5]; i++ ) {
 
             uint64_t a;
             MessageData tmp;
@@ -538,7 +641,7 @@ static void unpack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
         assert( FENCE == fence );
 
         /* unpack messageSpeedMapPerLength */
-        for ( uint32_t i= 0; i < sizes[5]; i++ ) {
+        for ( uint32_t i= 0; i < sizes[6]; i++ ) {
 
             uint64_t a;
             uint64_t b;
@@ -561,7 +664,7 @@ static void unpack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
         assert( FENCE == fence );
 
         /* unpack collectiveMapPerGroup */
-        for ( uint32_t i= 0; i < sizes[6]; i++ ) {
+        for ( uint32_t i= 0; i < sizes[7]; i++ ) {
 
             uint64_t a;
             uint64_t b;
@@ -612,7 +715,7 @@ static void unpack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
     if ( alldata.params.clustering.enabled ) {
 
         /* unpack functionMapPerRank */
-        for ( uint32_t i= 0; i < sizes[7]; i++ ) {
+        for ( uint32_t i= 0; i < sizes[8]; i++ ) {
 
             uint64_t a;
             uint64_t b;
@@ -644,6 +747,31 @@ static void unpack_worker_data( AllData& alldata, uint32_t sizes[10] ) {
         MPI_Unpack( buffer, sizes[0], &position, &fence, 1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
         assert( FENCE == fence );
     }
+    
+    if ( alldata.params.create_marker ) {
+        
+        /* unpack functionMinMaxLocationMap */
+        for ( uint32_t i= 0;  i < sizes[9]; i++) {
+            
+            uint64_t a;
+            FunctionMinMaxLocationData tmp;
+            
+            MPI_Unpack( buffer, sizes[0], &position, (void*) &a,                     1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, (void*) &tmp.location.min,      1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, (void*) &tmp.location.max,      1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, (void*) &tmp.location.loc_min,  1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, (void*) &tmp.location.loc_max,  1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, (void*) &tmp.location.time_min, 1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            MPI_Unpack( buffer, sizes[0], &position, (void*) &tmp.location.time_max, 1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+            
+            alldata.functionMinMaxLocationMap[ a ].add(tmp);
+        }
+        
+        /* extra check that doesn't cost too much */
+        fence= 0;
+        MPI_Unpack( buffer, sizes[0], &position, &fence, 1, MPI_LONG_LONG_INT, MPI_COMM_WORLD );
+        assert( FENCE == fence );      
+    }
 }
 
 
@@ -758,3 +886,119 @@ bool ReduceData( AllData& alldata ) {
 
     return !error;
 }
+
+
+bool ReduceDataDispersion( AllData& alldata ) {
+    
+    bool error= false;
+    
+    assert( 1 < alldata.numRanks );
+    
+    /* start runtime measurement for reducing data */
+    StartMeasurement( alldata, 1, true, "reduce data dispersion" );
+    
+    VerbosePrint( alldata, 1, true, "reducing data dispersion\n" );
+    
+    /* implement reduction myself because MPI and C++ STL don't play with
+     each other */
+    
+    /* how many rounds until master has all the data? */
+    uint32_t num_rounds= Logi( alldata.numRanks ) -1;
+    uint32_t round_no= 0;
+    uint32_t round= 1;
+    while ( round < alldata.numRanks ) {
+        
+        round_no++;
+        
+        if ( 1 == alldata.params.verbose_level ) {
+            
+            VerbosePrint( alldata, 1, true, " round %u / %u\n",
+                         round_no, num_rounds );
+        }
+        
+        uint32_t peer= alldata.myRank ^ round;
+        
+        /* if peer rank is not there, do nothing but go on */
+        if ( peer >= alldata.numRanks ) {
+            
+            round= round << 1;
+            continue;
+        }
+        
+        /* send to smaller peer, receive from larger one */
+        uint32_t sizes[10];
+        char* buffer;
+        
+        if ( alldata.myRank < peer ) {
+            
+            MPI_Status status;
+            
+            MPI_Recv( sizes, 10, MPI_UNSIGNED, peer, 4, MPI_COMM_WORLD,
+                     &status );
+            
+            // DEBUG
+            //cout << "    round " << round << " recv " << peer << "--> " <<
+            //alldata.myRank << " with " <<
+            //sizes[0] << " bytes, " <<
+            //sizes[1] << ", " <<
+            //sizes[2] << ", " <<
+            //sizes[3] << ", " <<
+            //sizes[4] << "" << endl << flush;
+            
+            buffer= prepare_worker_data( alldata, sizes );
+            
+            VerbosePrint( alldata, 2, false,
+                         "round %u / %u: receiving %u bytes from rank %u\n",
+                         round_no, num_rounds, sizes[0], peer );
+            
+            MPI_Recv( buffer, sizes[0], MPI_PACKED, peer, 5, MPI_COMM_WORLD,
+                     &status );
+            
+            unpack_worker_data( alldata, sizes );
+            
+        } else {
+            
+            /* don't reduce function map global twice */ 
+            alldata.functionMapGlobal.clear();
+            
+            buffer= pack_worker_data( alldata, sizes );
+            
+            // DEBUG
+            //cout << "    round " << round << " send " << alldata.myRank <<
+            //" --> " << peer << " with " <<
+            //sizes[0] << " bytes, " <<
+            //sizes[1] << ", " <<
+            //sizes[2] << ", " <<
+            //sizes[3] << ", " <<
+            //sizes[4] << "" << endl << flush;
+            
+            VerbosePrint( alldata, 2, false,
+                         "round %u / %u: sending %u bytes to rank %u\n",
+                         round_no, num_rounds, sizes[0], peer );
+            
+            MPI_Send( sizes, 10, MPI_UNSIGNED, peer, 4, MPI_COMM_WORLD );
+            
+            MPI_Send( buffer, sizes[0], MPI_PACKED, peer, 5,
+                     MPI_COMM_WORLD );
+            
+            /* every work has to send off its data at most once,
+             after that, break from the collective reduction operation */
+            break;
+        }
+        
+        round= round << 1;
+    }
+    
+    alldata.freePackBuffer();
+    
+    /* synchronize error indicator with workers */
+    /*SyncError( alldata, error );*/
+    
+    if ( !error ) {
+        
+        /* stop runtime measurement for reducing data */
+        StopMeasurement( alldata, true, "reduce data dispersion" );
+    }
+    
+    return !error;
+}
