@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2011, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -36,10 +36,41 @@
 
 #include "config.h"
 
-// disable OpenMP on NEC SX platforms to work around a compiler error
-#if (defined(HAVE_OMP) && HAVE_OMP) && defined(_SX)
-#	undef HAVE_OMP
-#endif // HAVE_OMP && _SX
+#if defined(HAVE_OMP) && HAVE_OMP
+  // disable OpenMP under the following circumstances:
+
+  // on NEC SX platforms (causes "internal compiler error")
+# if defined(_SX)
+#   undef HAVE_OMP
+
+  // on MacOS X using GCC < v4.5
+  // causes "undefined reference to ___builtin_expect()"
+  // (induced by assert()'s within OpenMP-parallel regions)
+# elif (defined(__APPLE__) && defined(__MACH__) && defined(__GNUC__) && \
+       (__GNUC__ < 4 || (__GNUC__ == 4 &&  __GNUC_MINOR__ < 5)))
+#   undef HAVE_OMP
+
+  // using Open64 < v4.2.4 (causes "internal compiler error")
+# elif defined(__OPEN64__)
+#   if !defined(__OPENCC__) || !defined(__OPENCC_MINOR__) || !defined(__OPENCC_PATCHLEVEL__)
+      // unknown compiler version; disable OpenMP to be on the safe side
+#     undef HAVE_OMP
+#   else
+      // __OPENCC_PATCHLEVEL__ can be empty; redefine it to 0
+#     if !(__OPENCC_PATCHLEVEL__ + 0)
+#       undef __OPENCC_PATCHLEVEL__
+#       define __OPENCC_PATCHLEVEL__ 0
+#     endif
+      // disable OpenMP, if compiler version is less than 4.2.4
+#     if __OPENCC__ < 4 || (__OPENCC__ == 4 && (__OPENCC_MINOR__ < 2 || (__OPENCC_MINOR__ == 2 && __OPENCC_PATCHLEVEL__ < 4)))
+#       undef HAVE_OMP
+#     endif
+#   endif
+# endif // __OPEN64__
+#endif // HAVE_OMP
+#if defined(HAVE_OMP) && HAVE_OMP
+# include <omp.h>
+#endif // HAVE_OMP
 
 #include "vt_filterc.h"
 #include "vt_otfhandler.h"
@@ -49,6 +80,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <assert.h>
 
 #ifdef VT_MPI
 #	define MASTER if( MyRank == 0 )
@@ -371,8 +403,6 @@ int main( int argc, char** argv ) {
 
 		uint64_t minbytes = 0, curbytes = 0, maxbytes = 0;
 
-		uint64_t pminbytestmp, pcurbytestmp, pmaxbytestmp;
-
 		/* *** Read the trace once without handling records -> get the min and maxbytes.
 		       Read the definitions
 		       Add all processes to the process2filter mapping *** */
@@ -406,6 +436,8 @@ int main( int argc, char** argv ) {
 			&fha, OTF_DEFFUNCTION_RECORD );
 
 #ifndef VT_MPI
+		uint64_t pminbytestmp, pcurbytestmp, pmaxbytestmp;
+
 		/* progress stuff */
 		OTF_Reader_setRecordLimit( preader, 0 );
 
@@ -418,9 +450,6 @@ int main( int argc, char** argv ) {
 		OTF_Reader_eventBytesProgress( preader, &pminbytestmp, &pcurbytestmp, &pmaxbytestmp );
 		minbytes += pminbytestmp;
 		maxbytes += pmaxbytestmp;
-#else // VT_MPI
-		/* needed to avoid compiler warnings */
-		pminbytestmp = pcurbytestmp = pmaxbytestmp = 0;
 #endif // VT_MPI
 		
 		/* defs */
