@@ -13,17 +13,11 @@
 #ifndef _VT_UNIFY_MARKERS_H_
 #define _VT_UNIFY_MARKERS_H_
 
-#include "config.h"
-
-#include "vt_inttypes.h"
-
 #include "vt_unify.h"
 #include "vt_unify_defs_recs.h"
 #include "vt_unify_lvector.hh"
 #include "vt_unify_tkfac.h"
 
-#include <set>
-
 //
 // MarkersC class
 //
