@@ -66,8 +66,8 @@ typedef struct
 {
   VTGen* gen;                       /**< trace file and buffer */
 
-  char  name[512];                  /**< thread name */
-  char  name_suffix[128];           /**< suffix of thread name */
+  char name[512];                   /**< thread name */
+  char name_suffix[128];            /**< suffix of thread name */
 
   int stack_level;                  /**< current call stack level */
   int stack_level_at_off;           /**< call stack level at trace off */
@@ -78,10 +78,11 @@ typedef struct
                                       VT_TRACE_OFF, or
                                       VT_TRACE_OFF_PERMANENT */
 
+  uint32_t tid;                     /**< associated thread id */
   uint32_t parent_tid;              /**< parent thread id */
   uint32_t child_num;               /**< number of child threads */
 
-  uint8_t is_virtual;               /**< flag: is virtual thread? (e.g. GPU) */
+  uint8_t  is_virtual;              /**< flag: is virtual thread? (e.g. GPU) */
 
 #if !defined(VT_DISABLE_RFG)
 
@@ -91,8 +92,10 @@ typedef struct
 
 #if (defined (VT_MPI) || defined (VT_HYB))
 
-  uint64_t mpicoll_next_matchingid; /**< matching id counter for MPI collective
-                                         operations. */
+  uint8_t mpi_tracing_enabled;       /**< actual mode of MPI tracing
+                                          operation */
+  uint64_t mpicoll_next_matchingid;  /**< matching id counter for MPI collective
+                                          operations */
 
 #endif /* VT_MPI || VT_HYB */
 
@@ -107,8 +110,9 @@ typedef struct
 
 #if (defined (VT_IOWRAP) || (defined(HAVE_MPI2_IO) && HAVE_MPI2_IO))
 
-  uint64_t io_next_matchingid;
-  uint64_t io_next_handle;
+  uint64_t io_next_matchingid;      /**< matching id counter for I/O
+                                         operations */
+  uint64_t io_next_handle;          /**< handle id counter for I/O operations */
 
 #endif
 
@@ -141,7 +145,10 @@ typedef struct
 
   void* plugin_cntr_defines;        /**< plugin cntr handle */
 
-#endif /* VT_PLUGIN_CNTR || VT_CUDARTWRAP */
+  uint8_t plugin_cntr_writing_post_mortem; /**< flag: writing post mortem
+                                                counter? */
+
+#endif /* VT_PLUGIN_CNTR */
 
 } VTThrd;
 
@@ -165,10 +172,13 @@ typedef struct
 #define VTTHRD_NAME_EXTERNAL(thrd)  (thrd->name_extern)
 
 /* parent thread id */
-#define VTTHRD_PARENT_TID(thrd)     (thrd->parent_tid);
+#define VTTHRD_PARENT_TID(thrd)     (thrd->parent_tid)
 
 /* number of child threads */
-#define VTTHRD_CHILD_NUM(thrd)      (thrd->child_num);
+#define VTTHRD_CHILD_NUM(thrd)      (thrd->child_num)
+
+/* flag: is virtual thread? */
+#define VTTHRD_IS_VIRTUAL(thrd)     (thrd->is_virtual)
 
 /* current call stack level */
 #define VTTHRD_STACK_LEVEL(thrd)    (thrd->stack_level)
@@ -186,39 +196,52 @@ typedef struct
 
 /* pop the call stack */
 #define VTTHRD_STACK_POP(thrd)      if(--(thrd->stack_level) < 0) \
-                                      vt_error_msg("Stack underflow");
+                                      vt_error_msg("Stack underflow")
+
+#if !defined(VT_DISABLE_RFG)
 
 /* RFG regions object */
 #define VTTHRD_RFGREGIONS(thrd)     (thrd->rfg_regions)
 
-/* flag: is virtual thread? */
-#define VTTHRD_IS_VIRTUAL(thrd)     (thrd->is_virtual)
+#endif /* VT_DISABLE_RFG */
 
 #if (defined (VT_MPI) || defined (VT_HYB))
-/* matching id counter for MPI coll. ops. */
+
+/* actual mode of MPI tracing operation */
+#define VTTHRD_MPI_TRACING_ENABLED(thrd) \
+                                    (thrd->mpi_tracing_enabled)
+
+/* increment matching id counter for MPI collective operations */
 #define VTTHRD_MPICOLLOP_NEXT_MATCHINGID(thrd) \
                                     (thrd->mpicoll_next_matchingid++)
+
 #endif /* VT_MPI || VT_HYB */
 
 #if (defined (VT_IOWRAP))
 
-/* save enabled/disabled state of I/O tracing when switching off temporarily */
+/* save value of enabled flag during suspend */
 #define VTTHRD_IO_TRACING_STATE(thrd) \
                                     (thrd->io_tracing_state)
+
+/* save how often suspend was called */
 #define VTTHRD_IO_TRACING_SUSPEND_CNT(thrd) \
                                     (thrd->io_tracing_suspend_cnt)
 
-/* flag: is I/O tracing enabled? */
+/* actual mode of I/O tracing operation */
 #define VTTHRD_IO_TRACING_ENABLED(thrd) \
                                     (thrd->io_tracing_enabled)
 
 #endif /* VT_IOWRAP */
 
 #if (defined (VT_IOWRAP) || (defined(HAVE_MPI2_IO) && HAVE_MPI2_IO))
+
+/* increment matching id counter for I/O operations */
 #define VTTHRD_IO_NEXT_MATCHINGID(thrd) \
                                     (thrd->io_next_matchingid++)
-#define VTTHRD_IO_NEXT_HANDLE(thrd) \
-                                    (thrd->io_next_handle++)
+
+/* increment handle id counter for I/O operations */
+#define VTTHRD_IO_NEXT_HANDLE(thrd) (thrd->io_next_handle++)
+
 #endif /* VT_IOWRAP || (HAVE_MPI2_IO && HAVE_MPI2_IO) */
 
 #if (defined (VT_GETCPU))
@@ -260,6 +283,10 @@ typedef struct
 #define VTTHRD_PLUGIN_CNTR_DEFINES(thrd) \
                                     (thrd->plugin_cntr_defines)
 
+/* flag: writing post mortem counter? */
+#define VTTHRD_PLUGIN_CNTR_WRITING_POST_MORTEM(thrd) \
+                                    (thrd->plugin_cntr_writing_post_mortem)
+
 #endif /* VT_PLUGIN_CNTR */
 
 
