@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -19,10 +20,13 @@
 #include "ompi_config.h"
 
 #include "ompi/mpi/c/bindings.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/runtime/params.h"
+#include "ompi/communicator/communicator.h"
+#include "ompi/errhandler/errhandler.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "ompi/memchecker.h"
 
-#if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Type_free = PMPI_Type_free
 #endif
 
@@ -43,7 +47,8 @@ int MPI_Type_free(MPI_Datatype *type)
    
    if( MPI_PARAM_CHECK ) {
       OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
-      if (NULL == type || NULL == *type || MPI_DATATYPE_NULL == *type) {
+      if (NULL == type || NULL == *type || MPI_DATATYPE_NULL == *type ||
+          ompi_datatype_is_predefined(*type)) {
         return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_TYPE,
                                       FUNC_NAME );
       }
@@ -51,7 +56,7 @@ int MPI_Type_free(MPI_Datatype *type)
 
    OPAL_CR_ENTER_LIBRARY();
 
-   rc = ompi_ddt_destroy( type );
+   rc = ompi_datatype_destroy( type );
    if( rc != MPI_SUCCESS ) {
       OMPI_ERRHANDLER_RETURN( MPI_ERR_INTERN, MPI_COMM_WORLD,
                              MPI_ERR_INTERN, FUNC_NAME );
