@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -17,34 +17,32 @@
  */
 
 #include "orte_config.h"
+#include "orte/constants.h"
 
 #include <stdio.h>
 
-#include "orte/orte_constants.h"
 #include "opal/util/trace.h"
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
+
+#include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/errmgr/base/base.h"
+#include "orte/mca/errmgr/base/errmgr_private.h"
 
 
 int orte_errmgr_base_close(void)
 {
     OPAL_TRACE(5);
     
-  /* If we have a selected component and module, then finalize it */
-
-  if (orte_errmgr_base_selected) {
-      orte_errmgr_base_selected_component.errmgr_finalize();
-  }
-
-  /* Close all remaining available components (may be one if this is a
-     OMPI RTE program, or [possibly] multiple if this is ompi_info) */
-
-  mca_base_components_close(orte_errmgr_base_output, 
-                            &orte_errmgr_base_components_available, NULL);
-
+    /* Close all remaining available components (may be one if this is a
+        OMPI RTE program, or [possibly] multiple if this is ompi_info) */
+    
+    mca_base_components_close(orte_errmgr_base_output, 
+                              &orte_errmgr_base_components_available, NULL);
+    
     orte_errmgr_initialized = false;
-  /* All done */
-
-  return ORTE_SUCCESS;
+    
+    /* All done */
+    
+    return ORTE_SUCCESS;
 }
