@@ -9,6 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -44,14 +46,113 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
                                      orte_app_context_t *app, orte_mapping_policy_t policy)
 {
     opal_list_item_t *item, *next;
-    orte_node_t *node;
+    orte_node_t *node, *nptr;
     orte_std_cntr_t num_slots;
     orte_std_cntr_t i;
     int rc;
+    opal_list_t nodes;
+    bool ignore;
 
     /** set default answer */
     *total_num_slots = 0;
     
+    /* if this is NOT a managed allocation, then we use the nodes
+     * that were specified for this app - there is no need to collect
+     * all available nodes and "filter" them
+     */
+    if (!orte_managed_allocation) {
+        OBJ_CONSTRUCT(&nodes, opal_list_t);
+        /* if the app provided a dash-host, then use those nodes */
+        if (NULL != app->dash_host) {
+            OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base.rmaps_output,
+                                 "%s using dash_host",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+            if (ORTE_SUCCESS != (rc = orte_util_add_dash_host_nodes(&nodes, &ignore,
+                                                                    app->dash_host))) {
+                ORTE_ERROR_LOG(rc);
+                return rc;
+            }
+        } else if (NULL != app->hostfile) {
+            /* otherwise, if the app provided a hostfile, then use that */
+            OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base.rmaps_output,
+                                 "%s using hostfile %s",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                 app->hostfile));
+            if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes, &ignore,
+                                                                   app->hostfile))) {
+                ORTE_ERROR_LOG(rc);
+                return rc;
+            }
+        } else if (NULL != orte_default_hostfile) {
+            /* fall back to the default hostfile, if provided */
+            OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base.rmaps_output,
+                                 "%s using default hostfile %s",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                 orte_default_hostfile));
+            if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes, &ignore,
+                                                                   orte_default_hostfile))) {
+                ORTE_ERROR_LOG(rc);
+                return rc;
+            }
+            /* this is a special case - we always install a default
+             * hostfile, but it is empty. If the user didn't remove it
+             * or put something into it, then we will have pursued that
+             * option and found nothing. This isn't an error, we just need
+             * to use the local host
+             */
+            if (0 == opal_list_get_size(&nodes)) {
+                OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base.rmaps_output,
+                                     "%s nothing in default hostfile - using local host",
+                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+                node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, 0);
+                OBJ_RETAIN(node);
+                opal_list_append(allocated_nodes, &node->super);
+                OBJ_DESTRUCT(&nodes);
+                goto complete;
+            }
+        } else {
+            /* if nothing else was available, then use the local host */
+            OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base.rmaps_output,
+                                 "%s using local host",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+            node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, 0);
+            OBJ_RETAIN(node);
+            opal_list_append(allocated_nodes, &node->super);
+            OBJ_DESTRUCT(&nodes);
+            goto complete;
+        }
+        /** if we still don't have anything */
+        if (0 == opal_list_get_size(&nodes)) {
+            orte_show_help("help-orte-rmaps-base.txt",
+                           "orte-rmaps-base:no-available-resources",
+                           true);
+            OBJ_DESTRUCT(&nodes);
+            return ORTE_ERR_SILENT;
+        }
+        /* find the nodes in our node array */
+        while (NULL != (item = opal_list_remove_first(&nodes))) {
+            nptr = (orte_node_t*)item;
+            for (i=0; i < orte_node_pool->size; i++) {
+                if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, i))) {
+                    continue;
+                }
+                if (0 != strcmp(node->name, nptr->name)) {
+                    continue;
+                }
+                /* retain a copy for our use in case the item gets
+                 * destructed along the way
+                 */
+                OBJ_RETAIN(node);
+                opal_list_append(allocated_nodes, &node->super);
+                OBJ_RELEASE(nptr);
+                break;
+            }
+        }
+        OBJ_DESTRUCT(&nodes);
+        /* now prune for usage and compute total slots */
+        goto complete;
+    }
+
     /* if the hnp was allocated, include it */
     if (orte_hnp_is_allocated) {
         node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, 0);
@@ -78,26 +179,6 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
         return ORTE_ERR_SILENT;
     }
     
-    /* is there a default hostfile? */
-    if (NULL != orte_default_hostfile) {
-        /* yes - filter the node list through the file, removing
-         * any nodes not in the file -or- excluded via ^
-         */
-        if (ORTE_SUCCESS != (rc = orte_util_filter_hostfile_nodes(allocated_nodes,
-                                                                  orte_default_hostfile))) {
-            ORTE_ERROR_LOG(rc);
-            return rc;
-        }
-        /** check that anything is here */
-        if (0 == opal_list_get_size(allocated_nodes)) {
-            orte_show_help("help-orte-rmaps-base.txt",
-                           "orte-rmaps-base:no-available-resources",
-                           true);
-            return ORTE_ERR_SILENT;
-        }
-    }
-    
-    
     /* did the app_context contain a hostfile? */
     if (NULL != app->hostfile) {
         /* yes - filter the node list through the file, removing
@@ -135,7 +216,6 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
         }
     }
     
-    
     /* now filter the list through any -host specification */
     if (NULL != app->dash_host) {
         if (ORTE_SUCCESS != (rc = orte_util_filter_dash_host_nodes(allocated_nodes,
@@ -192,6 +272,7 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
         }
     }
 
+ complete:
     /* remove all nodes that are already at max usage, and
      * compute the total number of allocated slots while
      * we do so
@@ -294,7 +375,7 @@ PROCESS:
 int orte_rmaps_base_claim_slot(orte_job_t *jdata,
                                orte_node_t *current_node,
                                int32_t cpus_per_rank,
-                               orte_std_cntr_t app_idx,
+                               int32_t app_idx,
                                opal_list_t *nodes,
                                bool oversubscribe,
                                bool remove_from_list,
@@ -321,9 +402,9 @@ int orte_rmaps_base_claim_slot(orte_job_t *jdata,
          */
         proc->app_idx = app_idx;
         OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base.rmaps_output,
-                             "%s rmaps:base:claim_slot: created new proc %s",
+                             "%s rmaps:base:claim_slot: created new proc %s for app_idx %d",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_NAME_PRINT(&proc->name)));
+                             ORTE_NAME_PRINT(&proc->name), app_idx));
         
         /* provide returned proc, if requested */
         if (NULL != returnproc) {
@@ -337,9 +418,9 @@ int orte_rmaps_base_claim_slot(orte_job_t *jdata,
     proc->nodename = current_node->name;
     
     OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base.rmaps_output,
-                         "%s rmaps:base:claim_slot mapping proc in job %s to node %s",
+                         "%s rmaps:base:claim_slot mapping proc in job %s app %d to node %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                         ORTE_JOBID_PRINT(jdata->jobid), current_node->name));
+                         ORTE_JOBID_PRINT(jdata->jobid), proc->app_idx, current_node->name));
     
     /* Be sure to demarcate the slots for this proc as claimed from the node */
     current_node->slots_inuse += 1;
@@ -390,48 +471,19 @@ int orte_rmaps_base_claim_slot(orte_job_t *jdata,
 int orte_rmaps_base_compute_vpids(orte_job_t *jdata)
 {
     orte_job_map_t *map;
-    orte_vpid_t vpid, vpid_start;
+    orte_vpid_t vpid, cnt;
     int i, j;
     orte_node_t *node;
-    orte_proc_t *proc;
+    orte_proc_t *proc, *ptr;
     int rc;
     
     map = jdata->map;
     
-    /* find the max vpid already assigned */
-    vpid_start = ORTE_VPID_MIN;
-    for (i=0; i < map->nodes->size; i++) {
-        if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
-            continue;
-        }
-        for (j=0; j < node->procs->size; j++) {
-            if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
-                continue;
-            }
-            /* ignore procs from other jobs */
-            if (proc->name.jobid != jdata->jobid) {
-                continue;
-            }
-            /* if the vpid is already defined, then update start */
-            if (ORTE_VPID_INVALID != proc->name.vpid &&
-                vpid_start < proc->name.vpid) {
-                vpid_start = proc->name.vpid;
-            }
-        }
-    }
-    if (ORTE_VPID_MIN == vpid_start) {
-        /* start at zero */
-        vpid_start = 0;
-    } else {
-        /* we start one higher than the max found */
-        vpid_start++;
-    }
-    
     if (ORTE_MAPPING_BYSLOT & map->policy ||
         ORTE_MAPPING_BYSOCKET & map->policy ||
         ORTE_MAPPING_BYBOARD & map->policy) {
         /* assign the ranks sequentially */
-        vpid = vpid_start;
+        vpid = 0;
         for (i=0; i < map->nodes->size; i++) {
             if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
                 continue;
@@ -445,12 +497,22 @@ int orte_rmaps_base_compute_vpids(orte_job_t *jdata)
                     continue;
                 }
                 if (ORTE_VPID_INVALID == proc->name.vpid) {
+                    /* find the next available vpid */
+                    while (NULL != (ptr = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, vpid)) &&
+                           ORTE_VPID_INVALID != ptr->name.vpid) {
+                        vpid++;
+                    }
                     proc->name.vpid = vpid++;
                 }
-                if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs,
-                                                                      proc->name.vpid, proc))) {
-                    ORTE_ERROR_LOG(rc);
-                    return rc;
+                /* some mappers require that we insert the proc into the jdata->procs
+                 * array, while others will have already done it - so check and
+                 * do the operation if required
+                 */
+                if (NULL == opal_pointer_array_get_item(jdata->procs, proc->name.vpid)) {
+                    if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs, proc->name.vpid, proc))) {
+                        ORTE_ERROR_LOG(rc);
+                        return rc;
+                    }                    
                 }
             }
         }
@@ -459,27 +521,57 @@ int orte_rmaps_base_compute_vpids(orte_job_t *jdata)
     
     if (ORTE_MAPPING_BYNODE & map->policy) {
         /* assign the ranks round-robin across nodes */
-        for (i=0; i < map->nodes->size; i++) {
-            if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
-                continue;
-            }
-            vpid = i + vpid_start;
-            for (j=0; j < node->procs->size; j++) {
-                if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
+        cnt=0;
+        vpid=0;
+        while (cnt < jdata->num_procs) {
+            for (i=0; i < map->nodes->size; i++) {
+                if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
                     continue;
                 }
-                /* ignore procs from other jobs */
-                if (proc->name.jobid != jdata->jobid) {
-                    continue;
-                }
-                if (ORTE_VPID_INVALID == proc->name.vpid) {
-                    proc->name.vpid = vpid;
-                    vpid += map->num_nodes;
-                }
-                if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs,
-                                                                      proc->name.vpid, proc))) {
-                    ORTE_ERROR_LOG(rc);
-                    return rc;
+                for (j=0; j < node->procs->size; j++) {
+                    if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
+                        continue;
+                    }
+                    /* ignore procs from other jobs */
+                    if (proc->name.jobid != jdata->jobid) {
+                        continue;
+                    }
+                    if (ORTE_VPID_INVALID != proc->name.vpid) {
+                        /* vpid was already assigned, probably by the
+                         * round-robin mapper. Some mappers require that
+                         * we insert the proc into the jdata->procs
+                         * array, while others will have already done it - so check and
+                         * do the operation if required
+                         */
+                        if (NULL == opal_pointer_array_get_item(jdata->procs, proc->name.vpid)) {
+                            if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs, proc->name.vpid, proc))) {
+                                ORTE_ERROR_LOG(rc);
+                                return rc;
+                            }
+                            /* if we added it to the array, then account for
+                             * it in our loop - otherwise don't as we would be
+                             * double counting
+                             */
+                            cnt++;
+                        }
+                        continue;
+                    }
+                    /* find next available vpid */
+                    while (NULL != (ptr = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, vpid)) &&
+                           ORTE_VPID_INVALID != ptr->name.vpid) {
+                        vpid++;
+                    }
+                    proc->name.vpid = vpid++;
+                    /* insert the proc into the jdata->procs array - can't already
+                     * be there as the only way to this point in the code is for the
+                     * vpid to have been INVALID
+                     */
+                    if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs, proc->name.vpid, proc))) {
+                        ORTE_ERROR_LOG(rc);
+                        return rc;
+                    }
+                    cnt++;
+                    break;  /* move on to next node */
                 }
             }
         }
